/* GETPARAM.C
 * The routine that implements the 'Get Parameter' message.
 *
 * Copyright (C) 1991 by Andrew Arensburger. Permission is granted to
 * use, copy and/or distribute this file freely for non-commercial purposes,
 * as long as this notice remains intact. Any commercial use is prohibited
 * without express permission from the author.
 *
 * If you make any changes, fix bugs, etc., please send them to me that I
 * might coordinate fixes and improvements.
 */

#include <stdio.h>
#include "eps.h"

/* GET_PARAM
 * Get the value of parameter 'par', for the instrument, layer, or
 * wavesample given in 'inst'.
 * The value of the parameter is returned in 'out'. This is a general
 * routine, so it is up to the calling routine to figure out how the
 * value of the parameter should be interpreted.
 * Returns 0 on no error, or a negative error code on error.
 */
int get_param(chan,inst,par,out)
int chan;			/* MIDI channel */
edit_spec *inst;		/* Which instrument? */
int par;			/* Which parameter? */
ulong *out;			/* Parameter value will be put here */
{
	int i;
	int waiting;		/* Are we waiting for the EPS to do things? */

	/* Send request for parameter value */
	send_head(chan);	/* Send SysEx head */
	mpu_sbyte(GET_PARAM);	/* Request parameter value */
	send12(inst->inst_num);	/* Instrument number */
	send12(inst->layer_num);/* Layer number */
	send12(inst->ws_num);	/* Wavesample number */
	send12((ushort) par);	/* Parameter number */
	send_tail();		/* Send SysEx tail */

	if (timeout(SHORT_TIMEOUT))
		return(-1);	/* Short timeout occurred */

	/* This loop is a bit tricky. The basic idea is that the EPS can
	 * send out one of three responses at this point: ACK, WAIT, or
	 * something else.
	 * If it sends an ACK, then all is well, and we can go on to
	 * receive the data. If it sends a something else, then there's
	 * clearly an error, and we can abort with a clean conscience.
	 * If it sends a WAIT, however, then we have to wait for up to
	 * 30 seconds, during which time any of the above can happen,
	 * including a second, third, etc. WAIT message.
	 * If the EPS has sent a WAIT, it will send an ACK before
	 * sending the data (but this was implied by the above para-
	 * graphs).
	 */
	waiting = 1;
	while (waiting)
	{
		/* See which response the EPS sends, and react
		 * accordingly.
		 */
		switch(recv_response(NULL)) {
			case ACK:	/* Acknowledgement */
				waiting = 0;	/* Stop waiting */
				break;

			case WAIT:	/* Wait for up to 30 seconds */
				send_response(chan, ACK);
					/* Okay, I'll wait */
				/* Wait for a timeout */
				if (timeout(LONG_TIMEOUT))
					return(-3);	/* 30-sec timeout
							 * has occurred
							 */
				break;

			default:	/* Something's wrong */
				wait(BREATHER);
				return(-4);
		}
	}

	/* Receive the instrument parameters, part 1 */
	recv_head(NULL);	/* Read the head of the incoming message */
	mpu_rbyte();		/* Should return 0x11 */
	recv12();		/* Should return instrument number */
	recv12();		/* Should return layer number */
	recv12();		/* Should return wavesample number */
	recv12();		/* Should return parameter number */
	*out = recv12();	/* Parameter value high word */
	*out <<= 16;
	*out |= recv12();	/* Parameter value low word */
	recv_tail();		/* Read the tail */

	/* NB: no final ACK */

	wait(BREATHER);
	return(0);
}
