/* PUTINST.C
 * The routine that implements the 'Put Instrument' message.
 * This file is the standard implementation for two-part 'put' messages.
 *
 * Copyright (C) 1991 by Andrew Arensburger. Permission is granted to
 * use, copy and/or distribute this file freely for non-commercial purposes,
 * as long as this notice remains intact. Any commercial use is prohibited
 * without express permission from the author.
 *
 * If you make any changes, fix bugs, etc., please send them to me that I
 * might coordinate fixes and improvements.
 */

#include <stdio.h>
#include "eps.h"

/* PUT_INSTRUMENT
 * Set the parameters of the instrument described in 'inst' to the
 * values given in 'params'.
 * Returns 0 on no error, or a negative error code.
 */
int put_instrument(chan,inst,params)
int chan;			/* MIDI channel */
edit_spec *inst;		/* Which instrument? */
inst_par *params;		/* Instrument parameters */
{
	int i;
	int waiting;		/* Are we waiting for the EPS to do things? */

	/* Send instruction to await instrument parameters */
	send_head(chan);	/* Send SysEx head */
	mpu_sbyte(PUT_INST);	/* 'Put Instrument' message */
	send12(inst->inst_num);	/* Instrument number */
	send12(inst->layer_num);/* Layer number */
	send12(inst->ws_num);	/* Wavesample number */
	send_tail();		/* Send SysEx tail */

	if (timeout(SHORT_TIMEOUT))
		return(-1);	/* Short timeout occurred */

	/* This loop is a bit tricky. The basic idea is that the EPS can
	 * send out one of three responses at this point: ACK, WAIT, or
	 * something else.
	 * If it sends an ACK, then all is well, and we can go on to
	 * receive the data. If it sends a something else, then there's
	 * clearly an error, and we can abort with a clean conscience.
	 * If it sends a WAIT, however, then we have to wait for up to
	 * 30 seconds, during which time any of the above can happen,
	 * including a second, third, etc. WAIT message.
	 * If the EPS has sent a WAIT, it will send an ACK before
	 * sending the data (but this was implied by the above para-
	 * graphs).
	 */
	waiting = 1;
	while (waiting)
	{
		/* See which response the EPS sends, and react
		 * accordingly.
		 */
		switch(recv_response(NULL)) {
			case ACK:	/* Acknowledgement */
				waiting = 0;	/* Stop waiting */
				break;

			case WAIT:	/* Wait for up to 30 seconds */
				send_response(chan, ACK);
					/* Okay, I'll wait */
				/* Wait for a timeout */
				if (timeout(LONG_TIMEOUT))
					return(-3);	/* 30-sec timeout
							 * has occurred
							 */
				break;

			default:	/* Something's wrong */
				wait(BREATHER);
				return(-4);
		}
	}

	/* Send instrument parameters */
	send_head(chan);
	for (i = 0; i < 12; i++)	/* Send instrument name */
		send16(params->name[i] << 8);
	send16(params->midi_chan << 8);	/* Send MIDI channel */
	send16(params->midi_prog << 8);	/* Send MIDI program number */
	send16(params->midi_pres << 8);	/* Send MIDI pressure */
	send16(params->size);		/* Send instrument size */
	send16(params->key_dest << 8);	/* Send key destination */
	for (i = 0; i < 4; i++)		/* Send bitmaps of layers */
		send16(params->patches[i] << 8);
	send16(params->down_layers << 8);	/* Send key down layers */
	send16(params->up_layers << 8);	/* Send key up layers */
	for (i = 23; i < 26; i++)	/* Unused */
		send16(0x00);
	send16(params->low_key << 8);	/* Send key range: low key */
	send16(params->high_key << 8);	/* Send key range: high key */
	send16(params->transpose << 8);	/* Send transposition */
	for (i = 0; i < 16; i++)	/* Send pitch table offsets */
		send16(params->pitch_off[i] << 8);
	for (i = 0; i < 16; i++)	/* Send layer offsets */
		send16(params->layer_off[i] << 8);
	for (i = 0; i < 256; i++)	/* Send wavesample offsets */
		send16(params->ws_off[i] << 8);
	for (i = 317; i < 323; i++)	/* Unused */
		send16(0x00);
	send_tail();			/* Send end of SysEx */

	/* Receive response to see if everything went well */
	i = recv_response(NULL);

	wait(BREATHER);
	if (i == ACK)
		return(0);		/* All went well */
	else
		return(-1);		/* There was some error */
}
