/* WSTRUNC.C
 * The routine that implements the 'Truncate Wavesample' message.
 *
 * Copyright (C) 1991 by Andrew Arensburger. Permission is granted to
 * use, copy and/or distribute this file freely for non-commercial purposes,
 * as long as this notice remains intact. Any commercial use is prohibited
 * without express permission from the author.
 *
 * If you make any changes, fix bugs, etc., please send them to me that I
 * might coordinate fixes and improvements.
 */

#include <stdio.h>
#include "eps.h"

/* WS_TRUNC
 * Truncate the wavesample described in 'inst' and reclaim any data before
 * the beginning of the wavesample or after the end.
 * Returns 0 on no error, or a negative error code.
 */
int ws_trunc(chan,inst)
int chan;			/* MIDI channel */
edit_spec *inst;		/* Which wavesample/ */
{
	int i;
	int waiting;		/* Are we waiting for something? */

	/* Send instruction to truncate the wavesample */
	send_head(chan);	/* Send SysEx head */
	mpu_sbyte(TRUNC_WS);	/* 'Truncate Wavesample' message */
	send12(inst->inst_num);	/* Instrument number */
	send12(inst->layer_num);/* Layer number */
	send12(inst->ws_num);	/* Wavesample number */
	send_tail();		/* Send SysEx tail */

	if (timeout(SHORT_TIMEOUT))
		return(-1);	/* Short timeout occurred */

	/* This loop is a bit tricky. The basic idea is that the EPS can
	 * send out one of three responses at this point: ACK, WAIT, or
	 * something else.
	 * If it sends an ACK, then all is well, and we can go on to
	 * receive the data. If it sends a something else, then there's
	 * clearly an error, and we can abort with a clean conscience.
	 * If it sends a WAIT, however, then we have to wait for up to
	 * 30 seconds, during which time any of the above can happen,
	 * including a second, third, etc. WAIT message.
	 * If the EPS has sent a WAIT, it will send an ACK before
	 * sending the data (but this was implied by the above para-
	 * graphs).
	 */
	waiting = 1;
	while (waiting)
	{
		/* See which response the EPS sends, and react
		 * accordingly.
		 */
		switch(recv_response(NULL)) {
			case ACK:	/* Acknowledgement */
				waiting = 0;	/* Stop waiting */
				break;

			case WAIT:	/* Wait for up to 30 seconds */
				send_response(chan, ACK);
					/* Okay, I'll wait */
				/* Wait for a timeout */
				if (timeout(LONG_TIMEOUT))
					return(-3);	/* 30-sec timeout
							 * has occurred
							 */
				break;

			default:	/* Something's wrong */
				wait(BREATHER);
				return(-4);
		}
	}

	/* If we have reached this point, then the last response
	 * received was an ACK.
	 */
	wait(BREATHER);
	return(0);		/* All went well */
}
