/* GETPTAB.C
 * The routine that implements the 'Get Pitch Table' message.
 *
 * Copyright (C) 1991 by Andrew Arensburger. Permission is granted to
 * use, copy and/or distribute this file freely for non-commercial purposes,
 * as long as this notice remains intact. Any commercial use is prohibited
 * without express permission from the author.
 *
 * If you make any changes, fix bugs, etc., please send them to me that I
 * might coordinate fixes and improvements.
 */

#include <stdio.h>
#include "eps.h"

/* GET_PTAB
 * Get the pitch table for the layer described in 'inst'.
 * The parameters are returned by filling in the fields in 'out'.
 * Returns 0 on no error, or a negative error code on error.
 * NB: I have no idea whether this works or not. About all I know
 * is that this routine receives the correct number of data bytes,
 * in approximately the correct manner. However, I don't know what
 * a pitch table is, how it can be modified, or what it does.
 */
int get_ptab(chan,inst,out)
int chan;			/* MIDI channel */
edit_spec *inst;		/* Which layer? */
pitch_tab *out;			/* Description will be put here */
{
	int i;
	int waiting;		/* Are we waiting for the EPS to do things? */

	/* Send request for pitch table */
	send_head(chan);	/* Send SysEx head */
	mpu_sbyte(GET_PITCHTAB);/* Request pitch table */
	send12(inst->inst_num);	/* Instrument number */
	send12(inst->layer_num);/* Layer number */
	send12(inst->ws_num);	/* Wavesample number */
	send_tail();		/* Send SysEx tail */

	if (timeout(SHORT_TIMEOUT))
		return(-1);	/* Short timeout occurred */

	/* This loop is a bit tricky. The basic idea is that the EPS can
	 * send out one of three responses at this point: ACK, WAIT, or
	 * something else.
	 * If it sends an ACK, then all is well, and we can go on to
	 * receive the data. If it sends a something else, then there's
	 * clearly an error, and we can abort with a clean conscience.
	 * If it sends a WAIT, however, then we have to wait for up to
	 * 30 seconds, during which time any of the above can happen,
	 * including a second, third, etc. WAIT message.
	 * If the EPS has sent a WAIT, it will send an ACK before
	 * sending the data (but this was implied by the above para-
	 * graphs).
	 */
	waiting = 1;
	while (waiting)
	{
		/* See which response the EPS sends, and react
		 * accordingly.
		 */
		switch(recv_response(NULL)) {
			case ACK:	/* Acknowledgement */
				waiting = 0;	/* Stop waiting */
				break;

			case WAIT:	/* Wait for up to 30 seconds */
				send_response(chan, ACK);
					/* Okay, I'll wait */
				/* Wait for a timeout */
				if (timeout(LONG_TIMEOUT))
					return(-3);	/* 30-sec timeout
							 * has occurred
							 */
				break;

			default:	/* Something's wrong */
				wait(BREATHER);
				return(-4);
		}
	}

	/* Receive the pitch table, part 1 */
	recv_head(NULL);	/* Read the head of the incoming message */
	mpu_rbyte();		/* Should return 0x10 */
	recv12();		/* Should return instrument number */
	recv12();		/* Should return layer number */
	recv12();		/* Should return wavesample number */
	recv_tail();		/* Read the tail */

	/* Send ACK to receive pitch table, Part 2 */
	send_response(chan, ACK);	/* Ask for next part */

	/* Receive the pitch table (Part 2: the actual table) */
	recv_head(NULL);
	for (i = 0; i < 12; i++)	/* Receive table name */
		out->name[i] = recv16() >> 8;
	for (i = 0; i < 88; i++)	/* Receive each entry */
	{
		ushort raw;		/* Raw data. This will be divided
					 * into semitones and fractions.
					 */

		raw = recv16();				/* Get raw data */
		out->semi[i] = raw >> 9;		/* Semitone */
		out->fract[i] = (raw >> 3) & 0x3f;	/* Fraction */
	}
	for (i = 100; i < 107; i++)
		recv16();		/* Unused */
	recv_tail();			/* Receive end of SysEx */

	/* Send ACK to EPS to say that everything has been received
	 * successfully.
	 */
	send_response(chan, ACK);

	wait(BREATHER);
	return(0);
}
